#!/usr/bin/perl

# Detect whether FTP server supports recursive ls and update flag in database
# Copyright (c) 2002-2003 Josef Zlomek
#
# This file is part of Fast File Search (FFS).
#
# FFS is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA,
# or download it from http://www.gnu.org/licenses/gpl.html

use strict;
use DBI;
use Getopt::Long qw(:config bundling no_ignore_case);
use Socket;
use Net::FTP;

# include the common package
$_ = $0;
$_ =~ s/[^\/]*$/common.pl/;
require;

my ($help, $private);
GetOptions ('help|h' => \$help,
		'common|master|c' => sub { $private = 0 },
		'private|slave|p' => \$private,
		);
if ($help)
{
	&help ();
	exit 0;
}

my %c;
my $db;

&read_config (\%c);
if ($private)
{
	&db_connect (\%c, undef, \$db);
}
else
{
	&db_connect (\%c, \$db);
}
exit 1 if (!$db);

my $servers = $db->selectall_arrayref ("SELECT ftp.HID,HostName,Port,Login,PassWord,Flags FROM ftp LEFT JOIN host USING (HID) WHERE HostName>'KO' ORDER BY HostName");
foreach my $server (@$servers)
{
	my $passive = 0;
	$passive = 1 if ($server->[5] & $c{HOST_PASSIVE_MODE});

	# try to connect to FTP server
	my $ftp_handle = Net::FTP->new ($server->[1], Port => $server->[2], Timeout => $c{FTP_TIMEOUT}, Passive => $passive);
	next if (!defined $ftp_handle);

	# get the server identification
	my $message = $ftp_handle->message ();

	if ($message =~ /PureFTPd/)
	{
		# PureFTPd sends only 2000 lines
		&clear_ls_recursive_flag ($server->[0]);

		$ftp_handle->close ();
		next;
	}

	if ($server->[4] eq 'user@host.com')
	{
		$server->[4] = $c{FTP_ANON_PASSWD};
	}
	if (!$ftp_handle->login ($server->[3], $server->[4]))
	{
		$ftp_handle->close ();
		next;
	}

	my $out_short = $ftp_handle->dir ();
	my $out_long = $ftp_handle->dir ("-lR");
	
	$ftp_handle->close ();

	next if (!$out_short || !$out_long);
	if (@$out_short < @$out_long)
	{
		&set_ls_recursive_flag ($server->[0]);
	}
	else
	{
		&clear_ls_recursive_flag ($server->[0]);
	}
}
&db_disconnect ($db);

sub clear_ls_recursive_flag
{
	my ($HID) = @_;

	$db->do ("UPDATE host SET Flags=Flags&~$c{HOST_LS_RECURSIVE} WHERE HID=$HID");
}

sub set_ls_recursive_flag
{
	my ($HID) = @_;

	$db->do ("UPDATE host SET Flags=Flags|$c{HOST_LS_RECURSIVE} WHERE HID=$HID");
}

sub help
{
	print "Fast File Search detect_recursive_ls.pl";
	print "  - detect whether FTP server supports recursive ls\n";
	print "Usage: detect_recursive_ls.pl.pl [OPTION]...\n\n";
	print "\n";
	print "    -c, --common, --master   Use common (master) database (default)\n";
	print "    -p, --private, --slave   Use private (slave) database\n";
	print "    -h, --help               Display this help and exit\n\n";
}
